local unusedBuffObjects =  setmetatable({}, {__mode='k'})
local unusedButtonObjects =  setmetatable({}, {__mode='k'})
local activeBuffObjects = {["HELPFUL"] = {}, ["HARMFUL"] = {}}

local DEBUGMODE=(UnitName("player") == "Straton" and GetCVar("realmname") == "Die Silberne Hand") and true
local N_DUMMYBUTTONS = 32
local MAX_DUMMYBUTTONS = 200
local MAXBUFFS = 40

local L = AceLibrary("AceLocale-2.2"):new("Buffalo2")
local waterfall = AceLibrary:HasInstance("Waterfall-1.0") and AceLibrary("Waterfall-1.0")
local aura = AceLibrary("SpecialEvents-Aura-2.0")


Buffalo2 = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceDB-2.0", "AceEvent-2.0", "AceDebug-2.0")
Buffalo2:RegisterDB("Buffalo2DB")
-- Buffalo2:RegisterDefaults("profile", Buffalo2_DEFAULT_OPTIONS)
local slashOptions = {
    type = 'group',
    args={
        reset = {
            type='execute',
            name="Reset",
            desc="Reset",
            func = function() Buffalo2.Reset(Buffalo2) end

        },
        config = {
            type='execute',
            name="config",
            desc="Config",
            func = function()
                if(waterfall) then
                    waterfall:Open("Buffalo2")
                else
                    Buffalo2:Print(L["Please install Waterfall-1.0 to access the configuration dialog"])
                end
            end
        },
    }
}
Buffalo2:RegisterChatCommand({"/Buffalo2"}, slashOptions)
Buffalo2.Templates = {}
Buffalo2.ActiveContainers = {}
Buffalo2.BUTTON_SIZE = 30
Buffalo2.abacus = AceLibrary("Abacus-2.0")
Buffalo2.crayon = AceLibrary("Crayon-2.0")
Buffalo2.dewdrop = AceLibrary("Dewdrop-2.0")
Buffalo2.MAJOR_TICK_PERIOD = 1
Buffalo2.MINOR_TICK_PERIOD = 0.1

function Buffalo2:OnInitialize()
    self:CreateOptionsTable()
    if(waterfall) then
        waterfall:Register("Buffalo2",
                            "aceOptions", self.options,
                            "title", L["Buffalo2 Configuration"],
                            "treeLevels", 3);
    end
    
end

function Buffalo2:OnEnable()
    for templateName in pairs(self.Templates) do
        self:CreateTemplateOptions(templateName)
    end
    self:SetDebugging(DEBUGMODE)
    for name in pairs(Buffalo2.Templates) do
        if(not self.db.profile[name]) then
            self.db.profile[name] = {}
        end
        if(self.db.profile[name].enabled) then
            self:EnableContainer(name)
        end
    end
    self.fullyInitialized = true
    self:RefreshAll()

    self.mainHandEnchant = Buffalo2_Buff:new()
    self.mainHandEnchant:Set("MH")
    self.offHandEnchant = Buffalo2_Buff:new()
    self.offHandEnchant:Set("OH")

    self:DisableBlizzardBuffs()
    self:ShowAll()
    self:ScheduleRepeatingEvent("Buffalo2_Major_Tick", self.MAJOR_TICK_PERIOD, self.MAJOR_TICK_PERIOD)
    self:ScheduleRepeatingEvent("Buffalo2_Minor_Tick", self.MINOR_TICK_PERIOD, self.MINOR_TICK_PERIOD)
    self:RegisterEvents()
end

function Buffalo2:OnDisable()
    self:EnableBlizzardBuffs()
    self:UnregisterEvents()
    self:HideAll()
end

function Buffalo2:OnProfileEnable()
    self:HideAll()
    for name in pairs(Buffalo2.Templates) do
        if(not self.db.profile[name]) then
            self.db.profile[name] = {}
        end
    end
    self:ShowAll()
end

function Buffalo2:Reset()
    for name in pairs(Buffalo2.Templates) do
        self.db.profile[name] = {}
    end
    self:OnProfileEnable()
end

function Buffalo2:AddTemplate(template)
  local name = template.GetName()
  self.Templates[name] = template
  self:CreateTemplateOptions(name)
  if(not self.db.profile[name]) then
      self.db.profile[name] = {}
  end
  if(self.db.profile[name].enabled) then
      self:EnableContainer(name)
  end
  self.fullyInitialized = true
  self:RefreshAll()
end

-------------------------------------------------------------------------------
-- Events
-------------------------------------------------------------------------------

function Buffalo2:UnregisterEvents()
    self:UnregisterAllEvents()
end

function Buffalo2:RegisterEvents()
    self:RegisterEvent("SpecialEvents_UnitBuffGained")
    self:RegisterEvent("SpecialEvents_UnitBuffLost")
    self:RegisterEvent("SpecialEvents_UnitBuffCountChanged")
    self:RegisterEvent("SpecialEvents_UnitBuffRefreshed")

    self:RegisterEvent("SpecialEvents_UnitDebuffGained")
    self:RegisterEvent("SpecialEvents_UnitDebuffLost")
    self:RegisterEvent("SpecialEvents_UnitDebuffCountChanged")
    self:RegisterEvent("SpecialEvents_UnitDebuffRefreshed")

    self:RegisterEvent("SpecialEvents_PlayerItemBuffLost")
    self:RegisterEvent("SpecialEvents_PlayerItemBuffGained")
    self:RegisterEvent("SpecialEvents_PlayerItemBuffRefreshed")
end

-- Buffs
function Buffalo2:SpecialEvents_UnitBuffGained(unitid, name, index, count, texture, rank, duration, timeLeft, identifier)
  local buff = self:GetBuff(unitid, name, index, count, texture, rank, duration, timeLeft, dispelType, "HELPFUL", identifier)
  self:AddBuff(buff)
end

function Buffalo2:SpecialEvents_UnitBuffLost(unitid, name, count, texture, rank, duration, identifier)
  local buff = self:GetBuff(unitid, name, index, count, texture, rank, duration, timeLeft, dispelType, "HELPFUL", identifier)
  self:RemoveBuff(buff)
end

function Buffalo2:SpecialEvents_UnitBuffCountChanged(unitid, name, index, count, texture, rank, duration, timeLeft, identifier)
  local buff = self:GetBuff(unitid, name, index, count, texture, rank, duration, timeLeft, dispelType, "HELPFUL", identifier)
  buff:UpdateCountOnButtons()
end

function Buffalo2:SpecialEvents_UnitBuffRefreshed(unitid, name, index, count, texture, rank, duration, timeLeft, identifier)
  local buff = self:GetBuff(unitid, name, index, count, texture, rank, duration, timeLeft, dispelType, "HELPFUL", identifier)
end

-- Debuffs
function Buffalo2:SpecialEvents_UnitDebuffGained(unitid, name, count, dispelType, texture, rank, index, duration, timeLeft, identifier)
  local buff = self:GetBuff(unitid, name, index, count, texture, rank, duration, timeLeft, dispelType, "HARMFUL", identifier)
  self:AddBuff(buff)
end

function Buffalo2:SpecialEvents_UnitDebuffLost(unitid, name, count, dispelType, texture, rank, duration, identifier)
  local buff = self:GetBuff(unitid, name, index, count, texture, rank, duration, timeLeft, dispelType, "HARMFUL", identifier)
  self:RemoveBuff(buff)
end

function Buffalo2:SpecialEvents_UnitDebuffCountChanged(unitid, name, count, dispelType, texture, rank, index, duration, timeLeft, identifier)
  local buff = self:GetBuff(unitid, name, index, count, texture, rank, duration, timeLeft, dispelType, "HARMFUL", identifier)
  buff:UpdateCountOnButtons()
end

function Buffalo2:SpecialEvents_UnitDebuffRefreshed(unitid, name, count, dispelType, texture, rank, index, duration, timeLeft, identifier)
  local buff = self:GetBuff(unitid, name, index, count, texture, rank, duration, timeLeft, dispelType, "HARMFUL", identifier)
end

-- Item Buffs
function Buffalo2:SpecialEvents_PlayerItemBuffGained(name, rank, mh, timeLeft, charges)
  if(mh) then
    self.mainHandEnchant:Set("MH", name, nil, charges, texture, rank, nil, timeLeft)
    self:AddBuff(self.mainHandEnchant)
  else
    self.offHandEnchant:Set("OH", name, nil, charges, texture, rank, nil, timeLeft)
    self:AddBuff(self.offHandEnchant)
  end
end

function Buffalo2:SpecialEvents_PlayerItemBuffLost(name, rank, mh)
  if(mh) then
    self.mainHandEnchant:Set("MH")
    self.mainHandEnchant:ClearRenderList()
  else
    self.offHandEnchant:Set("OH")
    self.offHandEnchant:ClearRenderList()
  end
end

function Buffalo2:SpecialEvents_PlayerItemBuffRefreshed(name, rank, mh, timeLeft, charges)
  local buff
  if(mh) then
    buff = self.mainHandEnchant
  else
    buff = self.offHandEnchant
  end
  buff.count = charges
  buff.timeLeft = timeLeft
  buff:UpdateCountOnButtons()
end

-------------------------------------------------------------------------------
-- Buff and Button caching
-------------------------------------------------------------------------------
function Buffalo2:AddBuff(buff)
  local buffIsNeeded
  for containerID, container in pairs(self.ActiveContainers) do
    if(container:ContainsBuff(buff)) then
      container:AddBuff(buff)
      buffIsNeeded = true
    end
  end
  if(not buffIsNeeded) then
    self:DepositBuff(buff)
  end
end

function Buffalo2:RemoveBuff(buff)
  for containerID, container in pairs(self.ActiveContainers) do
    if(container:ContainsBuff(buff)) then
      container:RemoveBuff(buff)
    end
  end
  buff:Dispose()
end

function Buffalo2:GetBuff(unitid, name, index, count, texture, rank, duration, timeLeft, dispelType, filter, identifier)
  local activeBuffObjectsForUnit = activeBuffObjects[filter][unitid]
  local buff
  if(activeBuffObjectsForUnit and activeBuffObjectsForUnit[identifier]) then
    buff = activeBuffObjectsForUnit[identifier]
    buff.index = index
    buff.count = count
    buff.timeLeft = timeLeft
  else
    buff = next(unusedBuffObjects) or Buffalo2_Buff:new()
    unusedBuffObjects[buff] = nil
    buff:Set(unitid, name, index, count, texture, rank, duration, timeLeft, dispelType, filter, identifier)
    if(not activeBuffObjectsForUnit) then
      activeBuffObjects[filter][unitid] = {}
    end
    activeBuffObjects[filter][unitid][identifier] = buff
  end
  return buff
end

function Buffalo2:DepositBuff(buff)
  if(buff == self.mainHandEnchant or buff == self.offHandEnchant) then
    return
  end
  if(activeBuffObjects[buff.filter][buff.unitid][buff.identifier]) then
    activeBuffObjects[buff.filter][buff.unitid][buff.identifier] = nil
  end
  buff:Set(nil)
  unusedBuffObjects[buff] = true
end

function Buffalo2:GetButton(container)
  local button = next(unusedButtonObjects) or Buffalo2_Button:new(container)
  unusedButtonObjects[button] = nil
  button:SetContainer(container)
  return button
end

function Buffalo2:DepositButton(button)
  if(button.buff) then
    button.buff:RemoveButtonFromRenderList(button)
  end
  button:SetContainer(nil)
  unusedButtonObjects[button] = true
end


-------------------------------------------------------------------------------
-- Layouts
-------------------------------------------------------------------------------

function Buffalo2:EnableContainer(containerID)
    local container
    if(not self.Templates[containerID]) then
        return
    elseif(self.ActiveContainers[containerID]) then
        container = self.ActiveContainers[containerID]
    else
        local containerTemplate = self.Templates[containerID]
        local newContainer = Buffalo2_Container:new(containerID, containerTemplate)
        container = newContainer
        container.enabled = true
        self.ActiveContainers[containerID] = container
        self:CreateContainerOptions(containerID, container)
    end
    container:Show()
end

function Buffalo2:DisableContainer(containerID)
    local container = self.ActiveContainers[containerID]
    if(container) then
        container:PreDestroy()
        container.enabled = nil
        self.ActiveContainers[containerID] = nil
        self:CreateTemplateOptions(containerID)
    end
end

function Buffalo2:SetEnableContainer(v, containerID)
    if(v) then
        self:EnableContainer(containerID)
    else
        self:DisableContainer(containerID)
    end
end

function Buffalo2:ResetContainer(containerID)
    local container = self.ActiveContainers[containerID]
    self.db.profile[containerID] = {}
    if(container) then
        self.db.profile[containerID].enabled = true
        self.db.profile[containerID].xpos = 500
        container:Hide()
        container:Show()
    end
end

-------------------------------------------------------------------------------
-- Container Handling
-------------------------------------------------------------------------------
function Buffalo2:HideAll()
    for containerID, container in pairs(self.ActiveContainers) do
        container:Hide()
    end
end

function Buffalo2:ShowAll()
    for containerID, container in pairs(self.ActiveContainers) do
        if(not container.hidden) then
            container:Show()
        end
    end
end

function Buffalo2:RefreshAll()
    for containerID, container in pairs(self.ActiveContainers) do
        container:RearrangeButtons()
    end
end

function Buffalo2:SetLockForAll(v)
    self.db.profile.locked = v
    for containerID, container in pairs(self.ActiveContainers) do
        container:SetLock(v)
    end
end

function Buffalo2:CreateDummyButtons()
    local n = min(N_DUMMYBUTTONS, floor(MAX_DUMMYBUTTONS / #self.ActiveContainers))
    for containerID, container in pairs(self.ActiveContainers) do
        container:CreateDummyButtons(n)
    end
end

function Buffalo2:SetShowGhostButtons(v)
    self.db.profile.ShowGhosts = v
    for containerID, container in pairs(self.ActiveContainers) do
            container:SetShowGhostBuffFrames(v)
    end
end

-------------------------------------------------------------------------------
-- Blizzard buffs
-------------------------------------------------------------------------------
function Buffalo2:DisableBlizzardBuffs()
    BuffFrame:Hide()
    TemporaryEnchantFrame:Hide()
end

function Buffalo2:EnableBlizzardBuffs()
    BuffFrame:Show()
    TemporaryEnchantFrame:Show()
end

-------------------------------------------------------------------------------
-- Debugging
-------------------------------------------------------------------------------
function Buffalo2:PrintAnchor(anchor)
    if(not self:IsDebugging()) then return end
    self:Debug("Point: ", anchor.point)
    self:Debug("RelativeTo: ", anchor.relativeTo)
    self:Debug("RelativePoint: ", anchor.relativePoint)
    self:Debug("X-Offset:", anchor.xOffs)
    self:Debug("Y-Offset:", anchor.yOffs)
end

function Buffalo2:PrintBuff(buff)
    if(not self:IsDebugging()) then return end
    self:Debug("----------------------------------")
    if(not buff) then
        self:Debug("Buff is nil!")
        self:Debug("----------------------------------")
        return
    end
    self:Debug("Unit ID: ", buff.unitid or "nil")
    self:Debug("Name: ", buff.name or "nil")
    self:Debug("Index: ", buff.index or "nil")
    self:Debug("Apps:", buff.count or "nil")
    self:Debug("Type:", buff.debuffType or "nil")
    self:Debug("Texture:", buff.texture or "nil")
    self:Debug("Rank: ", buff.rank or "nil")
    self:Debug("TimeLeft: ", buff.timeLeft or "nil")
    self:Debug("Duration: ", buff.duration or "nil")
    self:Debug("----------------------------------")
end

function Buffalo2:PrintBuffsOfUnit(unitid, n)
        if(not self:IsDebugging()) then return end
        self:Debug("----------------------------------")
        for i = 1,n do

        end
        self:Debug("----------------------------------")
end

function Buffalo2:Test(position)
    if true then return end
    if(DeBuffalo2Button1Border) then
        local r,g,b,a = DeBuffalo2Button1Border:GetVertexColor()
        self:Debug(position, a)
        self.printnext=false
    end
end
