local aceOO = AceLibrary("AceOO-2.0")
Buffalo2_Buff = aceOO.Class("AceEvent-2.0")
local aura = AceLibrary("SpecialEvents-Aura-2.0")

local __eq =
    function(buff1, buff2)
        return (buff1.unitid == buff2.unitid and buff1.identifier == buff2.identifier and buff1.filter == buff2.filter)
    end

function Buffalo2_Buff.prototype:init()
    Buffalo2_Buff.super.prototype.init(self)
    self.buttons = {}
    local mt = getmetatable(self)
    mt["__eq"] = __eq
    setmetatable(self, mt)
end

function Buffalo2_Buff.prototype:Dispose()
  self:ClearRenderList()
  Buffalo2:DepositBuff(self)
end

function Buffalo2_Buff.prototype:Set(unitid, name, index, count, texture, rank, duration, timeLeft, dispelType, filter, identifier)
  self.unitid = unitid or false
  self.name = name or false
  self.index = index or false
  self.count = count or false
  self.texture = texture or false
  self.rank = rank or false
  self.duration = duration or false
  self.timeLeft = timeLeft or false
  self.dispelType = dispelType or false
  self.filter = filter or false
  self.identifier = identifier or false
  if(unitid == "OH" or unitid == "MH") then
    assert(not index and not filter, "Weapon Enchants don't have index or filter!")
  end
  if(timeLeft and timeLeft > 0) then
    self:RegisterEvent("Buffalo2_Major_Tick")
  end
end

function Buffalo2_Buff.prototype:UnregisterEvents()
  self:UnregisterAllEvents()
end

-------------------------------------------------------------------------------
-- Button rendering
-------------------------------------------------------------------------------
function Buffalo2_Buff.prototype:AddButtonToRenderList(button)
  local container = button.container
--   local this_containers_button = self.buttons[container]
--   if(this_containers_button and this_containers_button ~= button) then
--     self:RemoveButtonFromRenderList(this_containers_button)
--   end
  self.buttons[container] = button
  button:SetBuff(self)
end

function Buffalo2_Buff.prototype:RemoveButtonFromRenderList(button)
    self.buttons[button.container] = nil
    button:SetBuff(nil)
end

function Buffalo2_Buff.prototype:UpdateAllButtons()
  for container, button in pairs(self.buttons) do
      button:SetBuff(self)
  end
end

function Buffalo2_Buff.prototype:UpdateTimeOnButtons()
  for container, button in pairs(self.buttons) do
      button:UpdateTime()
  end
end

function Buffalo2_Buff.prototype:UpdateCountOnButtons()
  for container, button in pairs(self.buttons) do
      button:UpdateCount()
  end
end


function Buffalo2_Buff.prototype:ClearRenderList()
  for container, button in pairs(self.buttons) do
    assert(button.container, "Button has no container, cannot be removed from rendering list!")
    self:RemoveButtonFromRenderList(button)
    button.container:RemoveBuff(self)
  end
end

function Buffalo2_Buff.prototype:GetIndex()
  if(self.filter == "HELPFUL") then
    self.index = aura:UnitHasBuff(self.unitid, self.name, self.rank, self.texture)
  elseif(self.filter == "HARMFUL") then
    self.index = aura:UnitHasDebuff(self.unitid, self.name, self.rank, self.texture)
  end
  return self.index
end


function Buffalo2_Buff.prototype:ShowInTooltip()
  if(self.unitid == "player") then
    local buffid = GetPlayerBuff(self:GetIndex(), self.filter)
    GameTooltip:SetPlayerBuff(buffid)
  elseif(self.unitid == "MH") then
    GameTooltip:SetInventoryItem("player", 16)
  elseif(self.unitid == "OH") then
    GameTooltip:SetInventoryItem("player", 17)
  else
    self:GetIndex()
    if(self.filter == "HELPFUL") then
      GameTooltip:SetUnitBuff(self.unitid, self:GetIndex())
    elseif(self.filter =="HARMFUL") then
      GameTooltip:SetUnitDebuff(self.unitid, self:GetIndex())
    end
  end
end

function Buffalo2_Buff.prototype:IsCancelable()
    if(self.unitid == "player" and self.filter == "HELPFUL") then
        return true
    elseif(self.unitid == "MH" or self.unitid == "OH") then
        return true
    end
end

function Buffalo2_Buff.prototype:Cancel()
  if(self.unitid == "player") then
    local buffid = GetPlayerBuff(self:GetIndex(), self.filter)
    CancelPlayerBuff(buffid)
  elseif(self.unitid == "MH") then
    CancelItemTempEnchantment(1)
  elseif(self.unitid == "OH") then
    CancelItemTempEnchantment(2)
  end
end

local _GetPlayerBuff = GetPlayerBuff
local _GetPlayerBuffTimeLeft = GetPlayerBuffTimeLeft
local _GetWeaponEnchantInfo = GetWeaponEnchantInfo
local _UnitBuff = UnitBuff
local _UnitDebuff = UnitDebuff
function Buffalo2_Buff.prototype:GetExactTimeLeft()
  local unitid = self.unitid
  local filter = self.filter
  if(unitid == "player") then
    local buffid = _GetPlayerBuff(self:GetIndex(), filter)
    return _GetPlayerBuffTimeLeft(buffid)
  elseif(unitid == "MH") then
    local timeLeft
    local _, timeLeft = _GetWeaponEnchantInfo()
    return (timeLeft or 0)/1000
  elseif(unitid == "OH") then
    local timeLeft
    local _, _, _, _, timeLeft = _GetWeaponEnchantInfo()
    return (timeLeft or 0)/1000
  elseif(filter == "HELFPUL") then
    local timeLeft
    _, _, _, _, _, timeLeft = _UnitBuff(unit, i)
    return timeLeft
  elseif(filter == "HARMFUL") then
    local timeLeft
    _, _, _, _, _, _, timeLeft = _UnitDebuff(unit, i)
    return timeLeft
  end
end

-------------------------------------------------------------------------------
-- Event Handlers
-------------------------------------------------------------------------------
function Buffalo2_Buff.prototype:Buffalo2_Major_Tick()
  local timeLeft = self.timeLeft
  if(timeLeft and timeLeft > 0) then
    self.timeLeft = timeLeft - Buffalo2.MAJOR_TICK_PERIOD
  end
  self:UpdateTimeOnButtons()
end
