local aceOO = AceLibrary("AceOO-2.0")
Buffalo2_Button = aceOO.Class("AceEvent-2.0")

Buffalo2_Button.nButtons = 0

function Buffalo2_Button.prototype:init(container)
    Buffalo2_Button.super.prototype.init(self)
    self.anchor = {}

    Buffalo2_Button.nButtons = Buffalo2_Button.nButtons + 1
    self.container = container
    self:CreateFrame()
    self:SetContainer(container)
    if(self.container.ShowGhosts) then
        self:ShowGhost()
    else
        self:HideGhost()
    end
end

function Buffalo2_Button.prototype:SetContainer(container)
  local mt
  self.container = container
  if(container) then
    mt = {__index=    function(table, propName)
                            if(self.container.Buttons[propName] ~= nil) then
                                return self.container.Buttons[propName]
                            elseif(Buffalo2.Default.Buttons[propName] ~= nil) then
                                return Buffalo2.Default.Buttons[propName]
                            else
                                return Buffalo2_Button.prototype[propName]
                            end
                        end,
          __mode="k",
         }
    setmetatable(self, mt)
    self:SetParent(container.frame)
  else
    mt = {__index=    function(table, propName)
                            if(Buffalo2.Default.Buttons[propName] ~= nil) then
                                return Buffalo2.Default.Buttons[propName]
                            else
                                return Buffalo2_Button.prototype[propName]
                            end
                        end,
          __mode="k",
         }
    setmetatable(self, mt)
    self:SetBuff(nil)
    self:SetParent(nil)
  end
end

local _CreateFrame = CreateFrame
function Buffalo2_Button.prototype:CreateFrame()
    if(self.frame) then
        error("Frame already exists", 2)
        return
    end
    self.frame = _CreateFrame("Button", "Buffalo2_Button"..Buffalo2_Button.nButtons.."_Frame", self.container.frame, "Buffalo2_ButtonTemplate")
    self.icon  = getglobal(self.frame:GetName().."Icon")
    self.counttext = getglobal(self.frame:GetName().."Count")
    self.border = getglobal(self.frame:GetName().."_Border")

    self.timer = _CreateFrame("Frame",  "Buffalo2_Button"..Buffalo2_Button.nButtons.."_Timer", self.container.frame, "Buffalo2_TimerTemplate")
    self.timertext  = getglobal(self.timer:GetName().."Duration")

    self.label = _CreateFrame("Frame",  "Buffalo2_Button"..Buffalo2_Button.nButtons.."_Label", self.container.frame, "Buffalo2_LabelTemplate")
    self.labeltext = getglobal(self.label:GetName().."Label")

    self.ghostLabel = getglobal(self.frame:GetName().."_Ghost_Label")
    self.ghostTex = getglobal(self.frame:GetName().."_Ghost_Texture")

    self.frame:RegisterForClicks("RightButtonUp", "LeftButtonUp")
    self.frame.object = self
end

function Buffalo2_Button.prototype:RegisterEvents()
  if(self.Buffalo2_Minor_Tick) then
    self:RegisterEvent("Buffalo2_Minor_Tick")
  elseif(self:IsEventRegistered("Buffalo2_Minor_Tick")) then
    self:UnregisterEvent("Buffalo2_Minor_Tick")
  end
  if(self.Buffalo2_Major_Tick) then
    self:RegisterEvent("Buffalo2_Major_Tick")
  elseif(self:IsEventRegistered("Buffalo2_Major_Tick")) then
    self:UnregisterEvent("Buffalo2_Major_Tick")
  end
end

function Buffalo2_Button.prototype:SetParent(parent)
        self.frame:SetParent(parent)
        self.timer:SetParent(parent)
        self.label:SetParent(parent)
end

function Buffalo2_Button.prototype:OnEnter()
    if(self.buff) then
      GameTooltip:SetOwner(self.frame, "ANCHOR_BOTTOMRIGHT", 15, -25)
      self.buff:ShowInTooltip()
      --self.container:PrintBuffList()
    end
end

function Buffalo2_Button.prototype:SetBuff(buff)
    self.buff = buff
    if(not buff) then
        self:SetTexture(nil)
        self:SetTimerText("")
        self:SetCountText("")
        self:SetLabelText("")
        self:Hide();
        if(self.container and self.container.ShowGhosts) then
            self:ShowGhost()
        else
            self:HideGhost()
        end
        self:UnregisterAllEvents()
        return
    end
    self:SetTexture(buff.texture)
    if(buff.unitid == "MH") then
        self.ghostLabel:SetText("MH")
        self:SetTexture(self:GetWeaponEnchantIcon(buff))
    elseif(buff.unitid == "OH") then
        self.ghostLabel:SetText("OH")
        self:SetTexture(self:GetWeaponEnchantIcon(buff))
    end
    self:SetTimerText(self:GetDurationString(buff))
    if(buff.count and buff.count > 0) then
        self:SetCountText(buff.count)
    else
        self:SetCountText("")
    end
    if(buff.debuffType) then
        if(buff.debuffType ~= "") then
            debuffColor = DebuffTypeColor[buff.debuffType];
        else
            debuffColor = DebuffTypeColor["none"];
        end
        self:SetBorderColor(debuffColor.r, debuffColor.g, debuffColor.b, 1.0)
    else
        self:SetBorderColor(1, 1, 1, 0.0)
    end
    self:SetLabelText(self:GetLabelString(buff))
    self:SetAlpha(1.0)
    self:Show()
    self:OnSetBuff(buff)
    self:RegisterEvents()
end

function Buffalo2_Button.prototype:SetTimerText(text, ...)
  local timertext = self.timertext
  if(not text) then
    timertext:SetText("")
  else
    timertext:SetFormattedText(text, ...)
    local width = timertext:GetStringWidth()
    timertext:SetWidth(width + 5)
  end
end

function Buffalo2_Button.prototype:SetLabelText(text)
  local labeltext = self.labeltext
  labeltext:SetText(text)
  local width = labeltext:GetStringWidth()
  labeltext:SetWidth(width + 5)
end

function Buffalo2_Button.prototype:SetCountText(text)
  if(text == self.oldcountText) then return end
  self.oldcountText = text
  self.counttext:SetText(text)
end

function Buffalo2_Button.prototype:SetGhostText(text)
    self.ghostLabel:SetText(text)
end

function Buffalo2_Button.prototype:SetTexture(tex)
    self.icon:SetTexture(tex)
end

function Buffalo2_Button.prototype:SetBorderColor(r, g, b, a)
  local border = self.border
  border:SetVertexColor(r, g, b)
  border:SetAlpha(a)
end

function Buffalo2_Button.prototype:SetAlpha(a)
    if(a and a >= 0 and a <= 1) then
        self.frame:SetAlpha(a)
        self.timer:SetAlpha(a)
        self.label:SetAlpha(a)
    else
        Buffalo2:Debug("Alpha value out of range:", a)
    end
end


function Buffalo2_Button.prototype:Show()
  local buff = self.buff
  local timeLeft = buff and buff.timeLeft
  local container = self.container
    self.frame:Show()
    if(buff and timeLeft and timeLeft > 0) or (container and container.ShowGhosts) then
        self.timer:Show()
    end
    self.label:Show()
    self.border:Show()
end

function Buffalo2_Button.prototype:Hide()
    self.frame:Hide()
    self.timer:Hide()
    self.label:Hide()
    self.border:Hide()
end

function Buffalo2_Button.prototype:ShowGhost()
    local r,g,b,a = self:GetGhostColor()
    self.ghostTex:SetTexture(r, g, b, a)

    if(not self.buff) then
        self:Show()
        self:SetTimerText("00:00")
        self:SetLabelText("Dummy")
        self:SetBorderColor(1, 1, 1, 1)
    end
    self.timer:Show()
    self.ghostLabel:Show()
    self.ghostTex:Show()
    self.border:Show()
end

function Buffalo2_Button.prototype:HideGhost()
    if(not self.buff or not self.buff.unitid) then
        self:Hide()
        self:SetTimerText("")
        self:SetLabelText("")
    end
    self.ghostLabel:Hide()
    self.ghostTex:Hide()
end

function Buffalo2_Button.prototype:ToString()
    local frame = "Frame: "..(self.frame:GetName() or "nil").."\n"
end

function Buffalo2_Button.prototype:UpdateTime()
  local buff = self.buff
  if(not buff or not buff.unitid) then return end
  local timeLeft = buff.timeLeft
  if(timeLeft and timeLeft ~= 0) then
      self:SetTimerText(self:GetDurationString(buff))
  elseif(buff.unitid == "MH" or buff.unitid == "OH" ) then
      self:SetTimerText(self:GetDurationString(buff))
  else
      self.timer:Hide()
  end
end

function Buffalo2_Button.prototype:UpdateCount()
  local buff = self.buff
  if(not buff or not buff.unitid) then return end
  local count = buff.count
  local unitid = buff.unitid
  if(count and count > 0) then
      if(unitid == "MH" or unitid == "OH") then
          self:SetCountText(count)
      elseif(count > 1) then
          self:SetCountText(count)
      else
          self:SetCountText("")
      end
  else
      self:SetCountText("")
  end
end

function Buffalo2_Button.prototype:OnClick(button)
    if(button == "RightButton" and not IsModifierKeyDown()) then
        if(self.buff:IsCancelable()) then
            self.buff:Cancel()
        else
            self:OnButtonClick(button)
        end
    else
        self:OnButtonClick(button)
    end
end

function Buffalo2_Button.prototype:OnDragStart()
    if(not self.container.locked) then
        self.container:StartMoving()
    end
end

function Buffalo2_Button.prototype:OnDragStop()
    if(not self.container.locked) then
        self.container:StopMoving()
    end
end
