local aceOO = AceLibrary("AceOO-2.0")
Buffalo2_Container = aceOO.Class("AceEvent-2.0")
local aura = AceLibrary("SpecialEvents-Aura-2.0")
local L = AceLibrary("AceLocale-2.2"):new("Buffalo2")


local scanUnitsOnInit = {
    "player",
    "target",
    "pet",
    "party1",
    "party2",
    "party3",
    "party4",
    "party5",
}

Buffalo2_Container.nContainer = 0

function Buffalo2_Container.prototype:init(name, template)
    Buffalo2_Container.super.prototype.init(self)
    self:CreateFrame(name)
    Buffalo2_Container.nContainer = Buffalo2_Container.nContainer + 1
    self.name = name
    self.buttons = Buffalo2_Vector:new()
    self.buffs = Buffalo2_Vector:new()
    self.anchors = {}
    self.sizeX = 0
    self.sizeY = 0
    self.firstAnchor = {    point = "TOPRIGHT",
                relativeTo=self.frame,
                relativePoint="TOPRIGHT",
                xOffs=0,
                yOffs=0}
    local mt = {__index =
                  function(table, propName)
                      if(Buffalo2.db.profile[name][propName]~= nil) then
                          return Buffalo2.db.profile[name][propName]
                      elseif(template[propName] ~= nil) then
                          return template[propName]
                      elseif(Buffalo2.Default[propName] ~= nil) then
                          return Buffalo2.Default[propName]
                      else
                          return Buffalo2_Container.prototype[propName]
                      end
                  end,
                __newindex =
                    function(table, key, value)
                        Buffalo2.db.profile[name][key] = value
                    end,}
    setmetatable(self, mt)
    self:ScanUnits(scanUnitsOnInit)
    self:SetScale(self.scale)
    ticks = 0
end

function Buffalo2_Container.prototype:PreDestroy()
    self:Hide()
    for i, button in self.buttons:Iterator() do
        Buffalo2:DepositButton(button)
    end
end

function Buffalo2_Container.prototype:CreateFrame(name)
    if(self.frame) then
        error("Frame already exists", 2)
        return
    end
    if(not name) then
        name = "Buffalo2_Container"..Buffalo2_Container.nContainer.."_Frame"
    end
    if(getglobal(name)) then
        self.frame = getglobal(name)
    else
        self.frame = CreateFrame("Frame", name, UIParent, "Buffalo2_ContainerTemplate")
    end
    self.frame:Show()
    self.frame.object = this
end

function Buffalo2_Container.prototype:Hide()
    self.frame:Hide()
end

function Buffalo2_Container.prototype:Show()
    self:RefreshBuffs()
    self.frame:Show()
end

function Buffalo2_Container.prototype:SetVisible(v)
    self.hidden = not v
    if(v) then
        self:Show()
    else
        self:Hide()
    end
end

function Buffalo2_Container.prototype:AddButton()
    if(self.buttons:Size() >= self.maxIcons) then
        return
    end
    local button = Buffalo2:GetButton(self)
    self.buttons:Add(button)
    self:SetShowGhostBuffFrames(self.ShowGhosts)
    self:RearrangeButtons()
    button:OnButtonAdding()
    return button
end

function Buffalo2_Container.prototype:ReleaseButton()
    local button = self.buttons:RemoveLast()
    if(button) then
      button:OnButtonRemoval()
      Buffalo2:DepositButton(button)
      self:RearrangeButtons()
    end
end

function Buffalo2_Container.prototype:CreateDummyButtons(n)
    while(self.buttons:Size() < self.maxIcons and self.buttons:Size() < n) do
        self:AddButton()
    end
end

function Buffalo2_Container.prototype:SetLock(value)
    self.locked = value
    if(value) then
        self:StopMoving()
    end
end

-------------------------------------------------------------------------------
-- Buff appearance
-------------------------------------------------------------------------------
function Buffalo2_Container.prototype:SetShowGhostBuffFrames(value)
    self.ShowGhosts = value
    if(value) then
        for i, button in self.buttons:Iterator() do
            if(button.buff and button.buff.unitid == "MH") then
                button:SetGhostText("MH")
            elseif(button.buff and button.buff.unitid == "OH") then
                button:SetGhostText("OH")
            else
                button:SetGhostText(i)
            end
            button:ShowGhost()
        end
    else
        for i, button in self.buttons:Iterator() do
            button:HideGhost()
        end
    end
--  self:FillAnchors()
end

-------------------------------------------------------------------------------
-- Buff adding, removing and sorting
-------------------------------------------------------------------------------
function Buffalo2_Container.prototype:ScanUnits(units)
  for i, unitid in ipairs(units) do
    for name, index, count, texture, rank, duration, identifier in aura:BuffIter(unitid) do
      local buff = Buffalo2:GetBuff(unitid, name, index, count, texture, rank, duration, nil, nil, "HELPFUL", identifier)
      local timeLeft = buff:GetExactTimeLeft()
      buff.timeLeft = timeLeft
      if(self:ContainsBuff(buff)) then
        self:AddBuff(buff)
      else
        buff:Dispose()
      end
    end

    for name, count, dispelType, texture, rank, index, duration, identifier in aura:DebuffIter(unitid) do
      local buff = Buffalo2:GetBuff(unitid, name, index, count, texture, rank, duration, nil, dispelType, "HARMFUL", identifier)
      local timeLeft = buff:GetExactTimeLeft()
      buff.timeLeft = timeLeft
      if(self:ContainsBuff(buff)) then
        self:AddBuff(buff)
      else
        buff:Dispose()
      end
    end
  end
end

function Buffalo2_Container.prototype:AddBuff(buff)
  for i, vbuff in self.buffs:Iterator() do
    if(vbuff == buff) then
      return
    end
  end
  self.buffs:Add(buff)
  if(self.buffs:Size() > self.buttons:Size()) then
    self:AddButton()
  end
  self:RefreshBuffs()
end

function Buffalo2_Container.prototype:RemoveBuff(buff)
   Buffalo2:Debug("RemoveBuff called for ", buff.name, self.name)
    for i, vbuff in self.buffs:Iterator() do
        if (buff == vbuff) then
            self.buffs:RemoveByValue(vbuff)
            if(self.buffs:Size() < self.buttons:Size()) then
              self:ReleaseButton()
            end
            break
        end
    end
    self:RefreshBuffs()
end

function Buffalo2_Container.prototype:ApplyBuffs()
   for i, buff in self.buffs:Iterator() do
--     Buffalo2:Debug("~~~~~~ Assigning Buff: ~~~~~~~")
--     Buffalo2:PrintBuff(buff)
--     Buffalo2:Debug("to Button: ", i)
--     Buffalo2:Debug("~~~~~~  ~~~~~~~")
    if(i > self.maxIcons) then return end
    buff:AddButtonToRenderList(self.buttons:Get(i))
  end
end

function Buffalo2_Container.prototype:SortBuffs()
  self.buffs:Sort(function(b1, b2) return self:Comparator(b1, b2) end)
end

function Buffalo2_Container.prototype:RefreshBuffs()
    self:SortBuffs()
    self:ApplyBuffs()
    self:SetShowGhostBuffFrames(self.ShowGhosts)
end

-------------------------------------------------------------------------------
-- Anchorage
-------------------------------------------------------------------------------
function Buffalo2_Container.prototype:RearrangeButtons()
  self:FillAnchors()
  self:ApplyAnchors()
end

--[[
returns the needed information to determine which button is to be attached to which button in what way. wwwwwww
*anchorPoint - the anchorpoint of the current button
*relativePoint - the point where we attach it to
*switchAnchorPoint, switchRelativePoint - like above, but for when the row / column switches ("linebreak")
*limit - the number of columns / rows where to perform a switch
]]
function Buffalo2_Container.prototype:GetAnchorInfo()
    local anchorPoint, relativePoint
    local switchAnchorPoint, switchRelativePoint
    local limit
    if(self.growHorizontalFirst) then
        limit = self.cols
        if(self.growRight) then
            anchorPoint = "LEFT"
            relativePoint = "RIGHT"
        else
            anchorPoint = "RIGHT"
            relativePoint = "LEFT"
        end

        if(self.growUpwards) then
            switchAnchorPoint = "BOTTOM"
            switchRelativePoint = "TOP"
        else
            switchAnchorPoint = "TOP"
            switchRelativePoint = "BOTTOM"
        end
    else
        limit = self.rows
        if(self.growRight) then
            switchAnchorPoint = "LEFT"
            switchRelativePoint = "RIGHT"
        else
            switchAnchorPoint = "RIGHT"
            switchRelativePoint = "LEFT"
        end

        if(self.growUpwards) then
            anchorPoint = "BOTTOM"
            relativePoint = "TOP"
        else
            anchorPoint = "TOP"
            relativePoint = "BOTTOM"
        end
    end
    return anchorPoint, relativePoint, switchAnchorPoint, switchRelativePoint, limit
end

function Buffalo2_Container.prototype:ApplyAnchors()
    if(not Buffalo2.fullyInitialized) then return end
    self.frame:ClearAllPoints()
    local point, relativeTo, relativePoint, xOffs, yOffs = self:GetContainerAnchor()
    if(Buffalo2.ActiveContainers[relativeTo]) then
        relativeTo = Buffalo2.ActiveContainers[relativeTo].frame
    end
    self.frame:SetPoint(point, relativeTo, relativePoint, xOffs/self.frame:GetEffectiveScale(), yOffs/self.frame:GetEffectiveScale())
    for i, button in self.buttons:Iterator() do
        local anchor = button.anchor
        button.frame:ClearAllPoints()
        button.frame:SetPoint(anchor.point, anchor.relativeTo, anchor.relativePoint, anchor.xOffs, anchor.yOffs)
        button.timer:ClearAllPoints()
        local point, relativePoint, xOffs, yOffs = button:GetTimerAnchor()
        button.timer:SetPoint(point, button.frame, relativePoint, xOffs, yOffs)
        button.label:ClearAllPoints()
        local point, relativePoint, xOffs, yOffs = button:GetLabelAnchor()
        button.label:SetPoint(point, button.frame, relativePoint, xOffs, yOffs)
    end
end

function Buffalo2_Container.prototype:FillAnchors()
    self:AdjustFrame()
    for i, button in self.buttons:Iterator() do
        local anchor = button.anchor
        if(i == 1) then -- first button in this container
            anchor.point = self.firstAnchor.point
            anchor.relativeTo = self.firstAnchor.relativeTo
            anchor.relativePoint = self.firstAnchor.relativePoint
            anchor.xOffs = self.firstAnchor.xOffs
            anchor.yOffs = self.firstAnchor.yOffs
        else
            local anchorPoint, relativePoint, switchAnchorPoint, switchRelativePoint, limit = self:GetAnchorInfo()
            if(limit == 1) then
                anchor.point = switchAnchorPoint
                anchor.relativeTo = self.buttons:Get(i-1).frame
                anchor.relativePoint = switchRelativePoint
                anchor.xOffs = (not self.growHorizontalFirst and self:GetGrowRightCoeff()   * self.xpad) or 0
                anchor.yOffs = (    self.growHorizontalFirst and self:GetGrowUpwardsCoeff() * self.ypad) or 0
            elseif(mod(i-1, limit) == 0) then
                anchor.point = switchAnchorPoint
                anchor.relativeTo = self.buttons:Get(i-limit).frame
                anchor.relativePoint = switchRelativePoint
                anchor.xOffs = (not self.growHorizontalFirst and self:GetGrowRightCoeff()   * self.xpad) or 0
                anchor.yOffs = (    self.growHorizontalFirst and self:GetGrowUpwardsCoeff() * self.ypad) or 0
            else
                anchor.point = anchorPoint
                anchor.relativeTo = self.buttons:Get(i-1).frame
                anchor.relativePoint = relativePoint
                anchor.xOffs = (    self.growHorizontalFirst and self:GetGrowRightCoeff()   * self.xpad) or 0
                anchor.yOffs = (not self.growHorizontalFirst and self:GetGrowUpwardsCoeff() * self.ypad) or 0
            end
        end
    end
end

function Buffalo2_Container.prototype:GetGrowRightCoeff()
    if self.growRight then
        return 1
    else
        return -1
    end
    self:RearrangeButtons()
end

function Buffalo2_Container.prototype:GetGrowUpwardsCoeff()
    if self.growUpwards then
        return 1
    else
        return -1
    end
    self:RearrangeButtons()
end

--[[
Sets the distance between the buff buttons.
padding - the distance in pixels
direction - either "x" or "y"
]]
function Buffalo2_Container.prototype:SetPadding(padding, direction)
    self[direction .. "pad"] = padding
    self:RearrangeButtons()
end

function Buffalo2_Container.prototype:SetScale(scale)
        self.frame:SetScale(scale)
    self.scale = scale
    self:RearrangeButtons()
end

--[[
Sets the direction in which the buffs grow
direction - either "right", "left", "up" or "down"
]]
function Buffalo2_Container.prototype:SetGrowthDirection(direction)
    if(direction == "right") then
        self.growRight = true
    elseif(direction == "left") then
        self.growRight = false
    elseif(direction == "up") then
        self.growUpwards = true
    elseif(direction == "down") then
        self.growUpwards = false
    else
        assert(false, "Invalid argument for SetGrowthDirection: " .. direction", must be either left, right, up or down")
    end
    self:RearrangeButtons()
end

--[[
These set the number of rows/cols for this container
]]
function Buffalo2_Container.prototype:SetRows(rows)
    self.rows = rows
    self:RearrangeButtons()
end
function Buffalo2_Container.prototype:SetCols(cols)
    self.cols = cols
    self:RearrangeButtons()
end

function Buffalo2_Container.prototype:SetGrowHorizontalFirst(value)
    self.growHorizontalFirst = value or false
    self:RearrangeButtons()
end

function Buffalo2_Container.prototype:AdjustFrame()
    local nCols, nRows, nIcons
    if(self.ShowGhosts) then
        nIcons = self.buttons:Size()
    else
        nIcons = self.buffs:Size()
    end
    if(self.growHorizontalFirst) then
        nCols = min(self.cols, nIcons)
        nRows = ceil(nIcons / self.cols)
    else
        nRows = min(self.rows, nIcons)
        nCols = ceil(nIcons / self.rows)
    end
    self.sizeX = nCols * Buffalo2.BUTTON_SIZE + (nCols - 1) * self.xpad
    if(self.sizeX < 0) then
        self.sizeX = 0
    end
    self.sizeY = nRows * Buffalo2.BUTTON_SIZE + (nRows - 1) * self.ypad
    if(self.sizeY < 0) then
        self.sizeY = 0
    end
    self.frame:SetWidth(self.sizeX)
    self.frame:SetHeight(self.sizeY)
    if(self.growUpwards) then
        if(self.growRight) then
            self.firstAnchor.point = "BOTTOMLEFT"
        else
            self.firstAnchor.point = "BOTTOMRIGHT"
        end
    else
        if(self.growRight) then
            self.firstAnchor.point = "TOPLEFT"
        else
            self.firstAnchor.point = "TOPRIGHT"
        end
    end
--  self.ghostframe:ClearAllPoints()
--  self.ghostframe:SetPoint(firstAnchor.point, self.frame, firstAnchor.relativePoint)
    self.firstAnchor.relativePoint = self.firstAnchor.point
end

function Buffalo2_Container.prototype:SavePosition()
    local button = self.buttons:Get(1)
    if(not button) then
                return
        else
                button = button.frame
        end
    self.xpos = math.floor(button:GetLeft()*button:GetEffectiveScale() + Buffalo2.BUTTON_SIZE*button:GetEffectiveScale() / 2 + 0.5)
    self.ypos = math.floor(button:GetBottom()*button:GetEffectiveScale() + Buffalo2.BUTTON_SIZE*button:GetEffectiveScale() / 2 + 0.5)
    local x, y = GetCursorPosition()
end

function Buffalo2_Container.prototype:StartMoving()
    if((not self.locked) and self.GetContainerAnchor == Buffalo2.Default.GetContainerAnchor) then
        self.frame:SetMovable(true)
        self.frame:StartMoving()
    end
end

function Buffalo2_Container.prototype:StopMoving()
    self.frame:SetMovable(false)
    self.frame:StopMovingOrSizing()
    self:SavePosition()
end

-------------------------------------------------------------------
-- Debugging
-------------------------------------------------------------------
function Buffalo2_Container.prototype:PrintBuffList()
  for i, buff in self.buffs:Iterator() do
    Buffalo2:PrintBuff(buff)
  end
end
