local L = AceLibrary("AceLocale-2.2"):new("Buffalo2")

function Buffalo2:CreateOptionsTable()
    self.options = {
        type = "group",
        name = "Buffalo2",
        args = {
            ghost = {
                order = 20,
                name = L["Show Ghost Buffs"],
                type = "toggle",
                desc = L["Shows reference icons to position the buff icons while unbuffed. If you don't see anything (or not enough buttons), try using \" Create Dummy Button\"."],
                get =   function() 
                            return self.db.profile.ShowGhosts
                        end,
                set =   function(v)
                                self:SetShowGhostButtons(v)
                        end,
            },
            dummy = {
                order = 30,
                name = L["Create Dummy Buttons"],
                type = "execute",
                desc = L["Buffalo2's icons are generated dynamically (meaning, they will only be created once needed). This button will throw 16 buttons in each currently active container so you can change your layout while unbuffed"],
                func =  function(v)
                                self:CreateDummyButtons()
                        end,
            },
            reset = {
                order = 40,
                name = L["Reset All Containers"],
                type = "execute",
                desc = L["Resets all user customization. WARNING: ALL YOUR SETTINGS WILL BE LOST!"],
                func =  function(v)
                                self:Reset()
                        end,
            },
            containers =  {
                order = 50,
                name = L["Containers"],
                desc = L["Configure Containers"],
                type = "group",
                args = {},
            },
        }
    }
end

function Buffalo2:CreateContainerOptions(containerID, container)
    local name = container:GetName()
    self.options.args.containers.args[containerID] = {
        name = name,
        type = "group",
        desc = container:GetDescription(),
        icon = (self.ActiveContainers[containerID] and "Interface\\Icons\\Ability_Rogue_Shadowstrikes"),
--      icon =  function()
--                  if(self.ActiveContainers[containerID]) then
--                      return "Interface\\Icons\\Ability_Rogue_Shadowstrikes"
--                  end
--              end,
        args = {
            enabled = {
                order = 1,
                name = L["Enabled"],
                type = "toggle",
                desc = string.format(L["Enable %s"], name),
                get = function() return self.db.profile[containerID].enabled end,
                set = function(v) self:SetEnableContainer(v, containerID) end,
            },
            hide = {
                order = 20,
                name = L["Hide"],
                type = "toggle",
                desc = string.format(L["Hide %s"], name),
                get =   function() 
                            return container.hidden 
                        end,
                set =   function(v)
                            container:SetVisible(not v)
                        end,
                disabled =  function() return not Buffalo2.ActiveContainers[containerID] end,
            },
            lock = {
                order = 25,
                name = L["Lock"],
                type = "toggle",
                desc = string.format(L["Lock %s"], name),
                get =   function() 
                            return container.locked
                        end,
                set =   function(v)
                            container.locked = v
                        end,
                disabled =  function() return not Buffalo2.ActiveContainers[containerID] end,
            },
            vGrowth = {
                order = 30,
                name = L["Vertical Direction"],
                type = "text",
                desc = L["Should new buffs appear above or below of existing ones?"],
                get =   function()
                            if(container.growUpwards) then
                                return L["Upwards"]
                            else
                                return L["Downwards"]
                            end
                        end,
                set =   function(s)
                            if(s == L["Downwards"]) then
                                container:SetGrowthDirection("down")
                            else
                                container:SetGrowthDirection("up")
                            end
                        end,
                validate = {L["Upwards"], L["Downwards"]},
                disabled = function() return not Buffalo2.ActiveContainers[containerID] end,
            },
            hGrowth = {
                order = 40,
                name = L["Horizontal Direction"],
                type = "text",
                desc = L["Should new buffs appear to the right or to the left of existing ones?"],
                get =   function()
                            if(container.growRight) then
                                return L["Right"]
                            else
                                return L["Left"]
                            end
                        end,
                set =   function(s)
                            if(s == L["Right"]) then
                                container:SetGrowthDirection("right")
                            else
                                container:SetGrowthDirection("left")
                            end
                        end,
                validate = {L["Right"], L["Left"]},
                disabled = function() return not Buffalo2.ActiveContainers[containerID] end,
            },
            growHorizontalFirst = {
                order = 50,
                name = L["Growth Precedence"],
                type = "text",
                desc = L["In which direction should the buffs grow first (horizontally or vertically)?"],
                get =   function()
                            if(container.growHorizontalFirst) then
                                return L["Horizontal"]
                            else
                                return L["Vertical"]
                            end
                        end,
                set =   function(s)
                            container:SetGrowHorizontalFirst(s == L["Horizontal"])
                        end,
                validate = {L["Horizontal"], L["Vertical"]},
                disabled = function() return not Buffalo2.ActiveContainers[containerID] end,
            },
            scale = {
                order = 60,
                name = L["Scale"],
                type = "range",
                desc = L["Size of icons"],
                max = 10, min = 0.1, step = 0.1,
                get = function() return container.scale end,
                set = function(s) container:SetScale(s) end,
                disabled = function() return not Buffalo2.ActiveContainers[containerID] end,
            },
            xpad = {
                order = 70,
                name = L["X-Padding"],
                type = "range",
                desc = L["Set the vertical distance between the buff icons"],
                max = 100, min = -10, step = 1,
                get = function() return container.xpad end,
                set = function(s) container:SetPadding(s, "x") end,
                disabled = function() return not Buffalo2.ActiveContainers[containerID] end,
            },
            ypad = {
                order = 80,
                name = L["Y-Padding"],
                type = "range",
                desc = L["Set the horizontal distance between the buff icons"],
                max = 100, min = -10, step = 1,
                get = function() return container.ypad end,
                set = function(s) container:SetPadding(s, "y") end,
                disabled = function() return not Buffalo2.ActiveContainers[containerID] end,
            },
            rows = {
                order = 90,
                name = L["Rows"],
                type = "range",
                desc = L["Number of Rows. Only applies when Growth Precedence is Vertical"],
                max = 32, min = 1, step = 1,
                get = function() return container.rows end,
                set = function(s) container:SetRows(s) end,
                disabled = function() return not Buffalo2.ActiveContainers[containerID] end,
            },
            cols = {
                order = 100,
                name = L["Columns"],
                type = "range",
                desc = L["Number of Columns. Only applies when Growth Precedence is Horizontal"],
                max = 32, min = 1, step = 1,
                get = function() return container.cols end,
                set = function(s) container:SetCols(s) end,
                disabled = function() return not Buffalo2.ActiveContainers[containerID] end,
            },
            reset = {
                order = 110,
                name = L["Reset This Container"],
                type = "execute",
                desc = L["Resets all user customization. WARNING: ALL YOUR SETTINGS WILL BE LOST!"],
                func =  function(v)
                                self:ResetContainer(containerID)
                        end,
            },
        }
    }
end

function Buffalo2:CreateTemplateOptions(containerID)
    local name = Buffalo2.Templates[containerID].GetName()
    self.options.args.containers.args[containerID] = {
        name = name,
        type = "group",
        desc = Buffalo2.Templates[containerID].GetDescription(),
        args = {
            enabled = {
                order = 1,
                name = L["Enabled (enabling may require a reload of the UI)"],
                type = "toggle",
                desc = string.format(L["Enable %s"], name),
                get = function() return self.db.profile[containerID].enabled end,
                set = function(v) self:SetEnableContainer(v, containerID) end,
            },
            reset = {
                order = 40,
                name = L["Reset This Container"],
                type = "execute",
                desc = L["Resets all user customization. WARNING: ALL YOUR SETTINGS WILL BE LOST!"],
                func =  function(v)
                                self:ResetContainer(containerID)
                        end,
            },
        }
    }
end
