--------------------------------- Target.lua ---------------------------------

local function abbrev__index (self, key)
    local value = ""
    for c in key:gmatch("(%w)%w*") do
        value = value .. c
    end

    rawset(self, key, value)
    return value
end

local abbrev = setmetatable({}, { __index = abbrev__index })

Buffalo2.Templates.Target = {
    GetName = function(container)
        return "Target"
    end,

    GetDescription = function(container)
        return "Your target's debuffs"
    end,

    author = "Nimbal",

    growUpwards = false,
    growRight = false,
    growHorizontalFirst = false,
    rows = 4,
    cols = 3,
    xpad = 15,
    ypad = 15,
    scale = 1.0,
    xpos = 200,
    ypos = 200,

    ContainsBuff = function(container, buff)
      return buff.unitid == "target" and buff.filter == "HARMFUL"
    end,

    Buttons = {
        GetGhostColor = function(button)
            return 1.0, 1.0, 1.0, 0.7
        end,

        GetLabelString = function(button, buff)
            local name = buff.name
            return abbrev[name]
        end,

        OnSetBuff = function(button, buff)
            if(buff.duration and buff.timeLeft) then
                local startCooldownTime = GetTime()-(buff.duration-buff.timeLeft);
                CooldownFrame_SetTimer(button.cooldown, startCooldownTime, buff.duration, 1);
                button.cooldown:Show()
            else
                button.cooldown:Hide()
            end
        end,

        OnButtonAdding = function(button)
          if(not button.cooldown) then
            local cooldown = CreateFrame("Cooldown", button.frame:GetName().."_Cooldown", button.frame, "CooldownFrameTemplate")
            button.cooldown = cooldown
            cooldown:SetPoint("CENTER")
          end
          button.cooldown:Show()
        end,

        OnButtonRemoval = function(button)
          button.cooldown:Hide()
        end,
    },
}
